@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>                  *
- *   Copyright (C) 2004 by Albert Astals Cid <tsdgeos@terra.es>            *
+ *   Copyright (C) 2004-2006 by Albert Astals Cid <tsdgeos@terra.es>       *
  *                                                                         *
  *   With portions of code from kpdf/kpdf_pagewidget.cc by:                *
  *     Copyright (C) 2002 by Wilco Greven <greven@kde.org>                 *
@@ -66,7 +66,7 @@ public:
     QValueVector< PageViewItem * > items;
     QValueList< PageViewItem * > visibleItems;
 
-    // view layout (columns and continuous in KpdfSettings), zoom and mouse
+    // view layout (columns and continuous in Settings), zoom and mouse
     PageView::ZoomMode zoomMode;
     float zoomFactor;
     PageView::MouseMode mouseMode;
@@ -127,8 +127,8 @@ PageView::PageView( QWidget *parent, KPDFDocument *document )
     // create and initialize private storage structure
     d = new PageViewPrivate();
     d->document = document;
-    d->zoomMode = ZoomFixed;
-    d->zoomFactor = 1.0;
+    d->zoomMode = (PageView::ZoomMode)KpdfSettings::zoomMode();
+    d->zoomFactor = KpdfSettings::zoomFactor();
     d->mouseMode = MouseNormal;
     d->mouseMidStartY = -1;
     d->mouseOnRect = false;
@@ -171,6 +171,7 @@ PageView::PageView( QWidget *parent, KPDFDocument *document )
 
 PageView::~PageView()
 {
+    d->document->removeObserver( this );
     delete d;
 }
 
@@ -179,16 +180,19 @@ void PageView::setupActions( KActionCollection * ac )
     // Zoom actions ( higher scales takes lots of memory! )
     d->aZoom = new KSelectAction( i18n( "Zoom" ), "viewmag", 0, this, SLOT( slotZoom() ), ac, "zoom_to" );
     d->aZoom->setEditable( true );
+#if KDE_IS_VERSION(3,4,89)
+    d->aZoom->setMaxComboViewCount( 13 );
+#endif
     updateZoomText();
 
     KStdAction::zoomIn( this, SLOT( slotZoomIn() ), ac, "zoom_in" );
 
     KStdAction::zoomOut( this, SLOT( slotZoomOut() ), ac, "zoom_out" );
 
-    d->aZoomFitWidth = new KToggleAction( i18n("Fit to Page &Width"), "viewmagfit", 0, ac, "zoom_fit_width" );
+    d->aZoomFitWidth = new KToggleAction( i18n("Fit to Page &Width"), "view_fit_width", 0, ac, "zoom_fit_width" );
     connect( d->aZoomFitWidth, SIGNAL( toggled( bool ) ), SLOT( slotFitToWidthToggled( bool ) ) );
 
-    d->aZoomFitPage = new KToggleAction( i18n("Fit to &Page"), "viewmagfit", 0, ac, "zoom_fit_page" );
+    d->aZoomFitPage = new KToggleAction( i18n("Fit to &Page"), "view_fit_window", 0, ac, "zoom_fit_page" );
     connect( d->aZoomFitPage, SIGNAL( toggled( bool ) ), SLOT( slotFitToPageToggled( bool ) ) );
 
     d->aZoomFitText = new KToggleAction( i18n("Fit to &Text"), "viewmagfit", 0, ac, "zoom_fit_text" );
@@ -204,14 +208,14 @@ void PageView::setupActions( KActionCollection * ac )
     d->aViewContinuous->setChecked( KpdfSettings::viewContinuous() );
 
     // Mouse-Mode actions
-    d->aMouseNormal = new KRadioAction( i18n("&Normal"), "mouse", 0, this, SLOT( slotSetMouseNormal() ), ac, "mouse_drag" );
+    d->aMouseNormal = new KRadioAction( i18n("&Browse Tool"), "mouse", 0, this, SLOT( slotSetMouseNormal() ), ac, "mouse_drag" );
     d->aMouseNormal->setExclusiveGroup( "MouseType" );
     d->aMouseNormal->setChecked( true );
 
     KToggleAction * mz = new KRadioAction( i18n("&Zoom Tool"), "viewmag", 0, this, SLOT( slotSetMouseZoom() ), ac, "mouse_zoom" );
     mz->setExclusiveGroup( "MouseType" );
 
-    d->aMouseSelect = new KRadioAction( i18n("&Select"), "frame_edit", 0, this, SLOT( slotSetMouseSelect() ), ac, "mouse_select" );
+    d->aMouseSelect = new KRadioAction( i18n("&Select Tool"), "frame_edit", 0, this, SLOT( slotSetMouseSelect() ), ac, "mouse_select" );
     d->aMouseSelect->setExclusiveGroup( "MouseType" );
 
 /*    d->aMouseEdit = new KRadioAction( i18n("Draw"), "edit", 0, this, SLOT( slotSetMouseDraw() ), ac, "mouse_draw" );
@@ -271,7 +275,12 @@ void PageView::notifySetup( const QValueVector< KPDFPage * > & pageSet, bool doc
         // so pages are never relayouted
         QTimer::singleShot(0, this, SLOT(slotRelayoutPages()));
     else
+    {
+        // update the mouse cursor when closing because we may have close through a link and
+        // want the cursor to come back to the normal cursor
+        updateCursor( viewportToContents( mapFromGlobal( QCursor::pos() ) ) );
         resizeContents( 0, 0 );
+    }
 
     // OSD to display pages
     if ( documentChanged && pageSet.count() > 0 && KpdfSettings::showOSD() )
@@ -317,10 +326,19 @@ void PageView::notifyViewportChanged( bool smoothMove )
     const QRect & r = item->geometry();
     int newCenterX = r.left(),
         newCenterY = r.top();
-    if ( vp.reCenter.enabled )
+    if ( vp.rePos.enabled )
     {
-        newCenterX += (int)( vp.reCenter.normalizedCenterX * (double)r.width() );
-        newCenterY += (int)( vp.reCenter.normalizedCenterY * (double)r.height() );
+        if (vp.rePos.pos == DocumentViewport::Center)
+        {
+            newCenterX += (int)( vp.rePos.normalizedX * (double)r.width() );
+            newCenterY += (int)( vp.rePos.normalizedY * (double)r.height() );
+        }
+        else
+        {
+            // TopLeft
+            newCenterX += (int)( vp.rePos.normalizedX * (double)r.width() + viewport()->width() / 2 );
+            newCenterY += (int)( vp.rePos.normalizedY * (double)r.height() + viewport()->height() / 2 );
+        }
     }
     else
     {
@@ -408,11 +426,13 @@ bool PageView::canUnloadPixmap( int pageNumber )
 }
 //END DocumentObserver inherited methods
 
+
 void PageView::showText( const QString &text, int ms )
 {
     d->messageWindow->display(text, PageViewMessage::Info, ms );
 }
 
+
 //BEGIN widget events
 void PageView::viewportPaintEvent( QPaintEvent * pe )
 {
@@ -613,7 +633,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
         }
         return;
     }
-    else if( e->key() == '/' && d->document->isOpened() )
+    else if( e->key() == '/' && d->document->isOpened() && d->document->supportsSearching() )
     {
         // stop scrolling the page (if doing it)
         if ( d->autoScrollTimer )
@@ -661,8 +681,8 @@ void PageView::keyPressEvent( QKeyEvent * e )
                 // more optimized than document->setPrevPage and then move view to bottom
                 DocumentViewport newViewport = d->document->viewport();
                 newViewport.pageNumber -= 1;
-                newViewport.reCenter.enabled = true;
-                newViewport.reCenter.normalizedCenterY = 1.0;
+                newViewport.rePos.enabled = true;
+                newViewport.rePos.normalizedY = 1.0;
                 d->document->setViewport( newViewport );
             }
             break;
@@ -681,8 +701,8 @@ void PageView::keyPressEvent( QKeyEvent * e )
                 // more optmized than document->setNextPage and then move view to top
                 DocumentViewport newViewport = d->document->viewport();
                 newViewport.pageNumber += 1;
-                newViewport.reCenter.enabled = true;
-                newViewport.reCenter.normalizedCenterY = 0.0;
+                newViewport.rePos.enabled = true;
+                newViewport.rePos.normalizedY = 0.0;
                 d->document->setViewport( newViewport );
             }
             break;
@@ -777,7 +797,7 @@ void PageView::contentsMouseMoveEvent( QMouseEvent * e )
         case MouseZoom:
         case MouseSelect:
             // set second corner of selection
-            if ( (leftButton || d->aPrevAction) && !d->mouseSelectionRect.isNull() )
+            if ( leftButton || d->aPrevAction )
                 selectionEndPoint( e->x(), e->y() );
             break;
 
@@ -906,11 +926,12 @@ void PageView::contentsMouseReleaseEvent( QMouseEvent * e )
                     {
                         // handle click over a image
                     }
-                    else
-                    {
+                    // Enrico and me have decided this is not worth the trouble it generates
+                    // else
+                    // {
                         // if not on a rect, the click selects the page
-                        d->document->setViewportPage( pageItem->pageNumber(), PAGEVIEW_ID );
-                    }
+                        // d->document->setViewportPage( pageItem->pageNumber(), PAGEVIEW_ID );
+                    // }
                 }
             }
             else if ( rightButton )
@@ -956,6 +977,13 @@ void PageView::contentsMouseReleaseEvent( QMouseEvent * e )
             break;
 
         case MouseSelect:{
+
+            if (d->mouseSelectionRect.isNull() && rightButton) 
+	    {
+	    	 PageViewItem * pageItem = pickItemOnPoint( e->x(), e->y() );
+	    	 emit rightClick( pageItem ? pageItem->page() : 0, e->globalPos() );
+	    }
+	    
             // if a selection is defined, display a popup
             if ( (!leftButton && !d->aPrevAction) || (leftButton && d->aPrevAction) ||
                  d->mouseSelectionRect.isNull() )
@@ -1068,11 +1096,7 @@ void PageView::contentsMouseReleaseEvent( QMouseEvent * e )
                         QString error;
                         if (KApplication::startServiceByDesktopName("kttsd", QStringList(), &error))
                         {
-                            // FIXME: Since this will be backported to KDE 3.4 branch, don't display
-                            // error message as this would add a translated string.
-                            // Use kdebug for now.
-                            // d->messageWindow->display( i18n( "Starting KTTSD Failed") )
-                            kdDebug() << "Starting KTTSD Failed" << endl;
+                            d->messageWindow->display( i18n("Starting KTTSD Failed: %1").arg(error) );
                             KpdfSettings::setUseKTTSD(false);
                             KpdfSettings::writeConfig();
                         }
@@ -1141,8 +1165,8 @@ void PageView::wheelEvent( QWheelEvent *e )
             // more optmized than document->setNextPage and then move view to top
             DocumentViewport newViewport = d->document->viewport();
             newViewport.pageNumber += 1;
-            newViewport.reCenter.enabled = true;
-            newViewport.reCenter.normalizedCenterY = 0.0;
+            newViewport.rePos.enabled = true;
+            newViewport.rePos.normalizedY = 0.0;
             d->document->setViewport( newViewport );
         }
     }
@@ -1154,8 +1178,8 @@ void PageView::wheelEvent( QWheelEvent *e )
             // more optmized than document->setPrevPage and then move view to bottom
             DocumentViewport newViewport = d->document->viewport();
             newViewport.pageNumber -= 1;
-            newViewport.reCenter.enabled = true;
-            newViewport.reCenter.normalizedCenterY = 1.0;
+            newViewport.rePos.enabled = true;
+            newViewport.rePos.normalizedY = 1.0;
             d->document->setViewport( newViewport );
         }
     }
@@ -1273,6 +1297,7 @@ void PageView::updateItemSize( PageViewItem * item, int colWidth, int rowHeight
     {
         height = kpdfPage->ratio() * colWidth;
         item->setWHZ( colWidth, (int)height, (double)colWidth / width );
+        d->zoomFactor = (double)colWidth / width;
     }
     else if ( d->zoomMode == ZoomFitPage )
     {
@@ -1280,6 +1305,7 @@ void PageView::updateItemSize( PageViewItem * item, int colWidth, int rowHeight
         double scaleH = (double)rowHeight / (double)height;
         zoom = QMIN( scaleW, scaleH );
         item->setWHZ( (int)(zoom * width), (int)(zoom * height), zoom );
+        d->zoomFactor = zoom;
     }
 #ifndef NDEBUG
     else
@@ -1416,6 +1442,11 @@ void PageView::updateZoom( ZoomMode newZoomMode )
         d->aZoomFitWidth->setChecked( checkedZoomAction == d->aZoomFitWidth );
         d->aZoomFitPage->setChecked( checkedZoomAction == d->aZoomFitPage );
         d->aZoomFitText->setChecked( checkedZoomAction == d->aZoomFitText );
+        
+        // save selected zoom factor
+        KpdfSettings::setZoomMode(newZoomMode);
+        KpdfSettings::setZoomFactor(newFactor);
+        KpdfSettings::writeConfig();
     }
 }
 
@@ -1658,8 +1689,8 @@ void PageView::slotRelayoutPages()
                 int prevX = contentsX(),
                     prevY = contentsY();
                 const QRect & geometry = d->items[ vp.pageNumber ]->geometry();
-                double nX = vp.reCenter.enabled ? vp.reCenter.normalizedCenterX : 0.5,
-                       nY = vp.reCenter.enabled ? vp.reCenter.normalizedCenterY : 0.0;
+                double nX = vp.rePos.enabled ? vp.rePos.normalizedX : 0.5,
+                       nY = vp.rePos.enabled ? vp.rePos.normalizedY : 0.0;
                 center( geometry.left() + ROUND( nX * (double)geometry.width() ),
                         geometry.top() + ROUND( nY * (double)geometry.height() ) );
                 // center() usually moves the viewport, that requests pixmaps too.
@@ -1779,9 +1810,9 @@ void PageView::slotRequestVisiblePixmaps( int newLeft, int newTop )
     {
         // determine the document viewport
         DocumentViewport newViewport( nearPageNumber );
-        newViewport.reCenter.enabled = true;
-        newViewport.reCenter.normalizedCenterX = focusedX;
-        newViewport.reCenter.normalizedCenterY = focusedY;
+        newViewport.rePos.enabled = true;
+        newViewport.rePos.normalizedX = focusedX;
+        newViewport.rePos.normalizedY = focusedY;
         // set the viewport to other observers
         d->document->setViewport( newViewport , PAGEVIEW_ID);
     }
