@@ -338,6 +338,12 @@ KViewPart::KViewPart(QWidget *parentWidget, const char *widgetName, QObject *par
 
 KViewPart::~KViewPart()
 {
+  // Without the next two lines, konqueror crashes when it is quit
+  // while displaying a DVI file. I don't really understand
+  // why... --Stefan.
+  if (manager() != 0)
+    manager()->removePart(this);
+
   // Delete the partManager;
   setManager(0);
   delete partManager;
@@ -838,9 +844,6 @@ bool KViewPart::openFile()
     // Remember the name of the part. So only need to switch if really necessary.
     multiPageLibrary = service->library();
 
-    insertChild(multiPage);
-    removeChild(oldMultiPage);
-
     connect(partManager, SIGNAL(activePartChanged(KParts::Part*)), this, SIGNAL(pluginChanged(KParts::Part*)));
 
     // Switch to the new multiPage
@@ -854,10 +857,7 @@ bool KViewPart::openFile()
     // TODO: Get rid of this without causing nasty artifacts.
     kapp->processEvents();
     initializeMultiPage();
-
-    // This is a bit strange. We need to set the focus to the mainWidget because only then
-    // the KViewPart recieves a KParts::GUIActivateEvent which triggers the GUI merge.
-    mainWidget->setFocus();
+    partManager->setActivePart(this);
 
     // We disconnect because we dont want some FocusEvents to trigger a GUI update, which might mess
     // with our menus.
@@ -1349,14 +1349,19 @@ void KViewPart::readSettings()
   // not need to take any action here, because this method is called
   // only in the constructor of the KViewPart when no document is loaded.
   useDocumentSpecifiedSize->setChecked(KVSPrefs::useDocumentSpecifiedSize());
+
+  multiPage->readSettings();
 }
 
 
 void KViewPart::writeSettings()
 {
   // if loading the KPart failed - just exit now
-  if (!showSidebar)
-    return;
+  //if (!showSidebar)
+  //  return;
+
+  multiPage->writeSettings();
+
   KVSPrefs::setPageMarks(showSidebar->isChecked());
   KVSPrefs::setWatchFile(watchAct->isChecked());
   KVSPrefs::setZoom(_zoomVal.value());
